@@ -27,15 +27,16 @@ class PyuvmFormatter(SimColourLogFormatter):
 
 # 6.2.1
 class uvm_report_object(uvm_object):
+    __default_logging_level = logging.INFO
     """ The basis of all classes that can report """
-
     def __init__(self, name):
         super().__init__(name)
         uvm_root_logger = logging.getLogger('uvm')
         # Every object gets its own logger
         logger_name = self.get_full_name() + str(id(self))
         self.logger = uvm_root_logger.getChild(logger_name)
-        self.logger.setLevel(level=logging.INFO)  # Default is to print INFO
+        self.logger.setLevel(
+            level=uvm_report_object.get_default_logging_level())
         # We are not sending log messages up the hierarchy
         self.logger.propagate = False
         self._streaming_handler = logging.StreamHandler(sys.stdout)
@@ -44,10 +45,16 @@ class uvm_report_object(uvm_object):
         self._streaming_handler.setLevel(logging.NOTSET)
         # Make log messages look like UVM messages
         self._uvm_formatter = PyuvmFormatter(self.get_full_name())
-#        self._uvm_formatter = logging.Formatter(
-#            "%(levelname)s: %(filename)s(%(lineno)d)[" + self.get_full_name() + "]: %(message)s")  # noqa: E501
         self.add_logging_handler(self._streaming_handler)
 
+    @staticmethod
+    def set_default_logging_level(default_logging_level):
+        uvm_report_object.__default_logging_level = default_logging_level
+
+    @staticmethod
+    def get_default_logging_level():
+        return uvm_report_object.__default_logging_level
+
     def set_logging_level(self, logging_level):
         """ Sets the logger level """
         self.logger.setLevel(logging_level)
